#include <Constants.au3>

; Make sure that there is only one copy of the program running
; To do so, use the "AutoIt Window Title":

; Set a temporary Title to distinguish the new copy from the one that might already be running
AutoItWinSetTitle ( "NewWDSToggleWindow" )
If WinExists("WDSToggleWindow") Then
	WinClose("WDSToggleWindow")
	MsgBox(0,"Warning","A previous copy was already running!")
	Sleep(1000) ; Wait for the "old" copy of the program to Exit...
	; And then start the new one!
EndIf
AutoItWinSetTitle ( "WDSToggleWindow" ) ; Set the final title that this copy is already running

; Try to find the WDSDesktop window and get its handle
;Opt("WinSearchChildren",1)
While 1
  $WinHandle = WinGetHandle("Deskbar","")
  If $WinHandle Then ExitLoop ; The window was found, continue!
  ;TrayTip("Waiting for Windows Desktop Search to be loaded","",1000)
  Sleep(10000) ; Wait 10 secs and try to find the Deskbar window again!
WEnd
;MsgBox(0,"Debug","Found the WDS window!")
TrayTip("The WDS window was found!","",50000)
Global $s_startup_shortcut = @StartMenuCommonDir & "\Programs\Startup\WDSToggle.lnk"

Opt("TrayMenuMode",1)   ; Default tray menu items (Script Paused/Exit) will not be shown.
Opt("WinTitleMatchMode",4) ; Use "classname" to match windows as well
Opt("TrayOnEventMode",1); Enable Tray Events

Global $s_toggle_key
$s_toggle_key = RegRead("HKEY_LOCAL_MACHINE\SOFTWARE\WDSToggle","ToggleKey")
If $s_toggle_key = "" Then
	;MsgBox(0,"Debug","ToggleKey not set yet")
	$s_toggle_key = "{F8}"
	RegWrite("HKEY_LOCAL_MACHINE\SOFTWARE\WDSToggle","ToggleKey","REG_SZ",$s_toggle_key)
EndIf

Global $b_auto_resize
$b_auto_resize = RegRead("HKEY_LOCAL_MACHINE\SOFTWARE\WDSToggle","AutoResize")
If $b_auto_resize = "" Then
	MsgBox(0,"Debug","AutoResize not set yet")
	$b_auto_resize = 1
	RegWrite("HKEY_LOCAL_MACHINE\SOFTWARE\WDSToggle","AutoResize","REG_SZ",$b_auto_resize)
ElseIf $b_auto_resize = "1" Then
  $b_auto_resize = 1
Else
  $b_auto_resize = 0
EndIf


If $b_auto_resize = 1 Then
  ; Set the proper size when starting the Toggler!
  $v_pos = WinGetPos($WinHandle)
  WinMove($WinHandle, "", $v_pos[0], $v_pos[1], $v_pos[2], 62)
EndIf

HotkeySet($s_toggle_key,"WDSToggle") ; Show The WDS Desktop window when clicking F*...
TraySetOnEvent ( $TRAY_EVENT_PRIMARYUP, "WDSToggle" ) ; ... and when clicking on the tray icon

TraySetClick ( 8 ) ; Show the regular AutoIt tray meny when right clicking on the tray icon
TraySetToolTip("Search Desktop")
If @compiled Then
	Global $startup_item  = TrayCreateItem("Run at startup")
	TrayItemSetOnEvent ( -1, "ToggleStartupSetting" )
	; Make sure that the "Run at startup" element is properly checked or unchecked
	If CurrentStartupSetting() == True Then
		TrayItemSetState($startup_item, $TRAY_CHECKED)
	Else
		TrayItemSetState($startup_item, $TRAY_UNCHECKED)
	EndIf
EndIf
$auto_resize_item  = TrayCreateItem("Auto Resize WDS")
TrayItemSetOnEvent ( -1, "ToggleAutoResizeSetting")
; Make sure that the item is properly checked/unchecked
If $b_auto_resize = 1 Then
  TrayItemSetState(-1, $TRAY_CHECKED)
Else
  TrayItemSetState(-1, $TRAY_UNCHECKED)
EndIf

TrayCreateItem("") ; Menu Separator

Global $togglekey_item  = TrayCreateItem("Change Toggle Key combo (currently set to '" & $s_toggle_key & "')")
TrayItemSetOnEvent ( -1, "ChangeToggleKey" )

TrayCreateItem("") ; Menu Separator

$exit_item  = TrayCreateItem("Exit")
TrayItemSetOnEvent ( -1, "ExitProgram" )


Global $last_state = @SW_HIDE
; We need a loop to keep the HotKey alive!
While 1
	; Also, every minute ensure that the toolbar is kept in the last state
	; For some reason it keeps showing up!
	;WDSToggle($last_state)
	;Sleep(60000)
	Sleep(600000) ; WDS does not have this problem - Only wake up every 10 minutes!
WEnd

;- End of MAIN CODE ------------------------------------------------------------


;- FUNCTION DEFINITIONS --------------------------------------------------------

;- Function that shows/hides the WDS Desktop floatie
Func WDSToggle($new_state = -1)
  $s_wds_class = "Deskbar" ; Define the WDS Desktop window classname
	
  If WinExists($s_wds_class,"") == 0 Then
    ; Do nothing if WDS Desktop is not running!
    $s_wds_class = "Deskbar"
    ;If WinExists($s_wds_class,"") == 0 Then
		;MsgBox(0,"Debug","Error","GD not found!")
		Return
    ;EndIf
  EndIf
  
  If @NUMPARAMS == 0 Then
	  ; When passing no new state, swap the the current state
	  $new_state = -1
  EndIf
  
  If $new_state == @SW_HIDE OR BitAND(2,WinGetState($s_wds_class)) Then
	 ; If it is visible, hide it
    ControlSend($s_wds_class,"","Edit1","{Esc}{BACKSPACE}{Esc}")
    WinSetState($s_wds_class,"",@SW_HIDE)
    $last_state = @SW_HIDE
    TraySetToolTip("Show Desktop Search")
  Else
    WinSetState($s_wds_class,"",@SW_SHOW)
    WinActivate($s_wds_class,"")
    Sleep(100)
    ControlClick($s_wds_class,"","Edit1")
    $last_state = @SW_SHOW
    TraySetToolTip("Hide Desktop Search")
  EndIf
EndFunc

Func CurrentStartupSetting()
	If FileExists($s_startup_shortcut) Then
		$b_run_on_startup = True
	Else
		$b_run_on_startup = False
	EndIf
	;MsgBox(0,"Debug","$b_run_on_startup = " & $b_run_on_startup)
	
	Return $b_run_on_startup
EndFunc

Func ToggleStartupSetting()
	If CurrentStartupSetting() == True Then
		;MsgBox(0,"Debug","Deleting Shortcut")
		FileDelete($s_startup_shortcut)
		TrayItemSetState($startup_item, $TRAY_UNCHECKED)
	Else
		;MsgBox(0,"Debug","Creating Shortcut")
		;MsgBox(0,"Debug",$s_startup_shortcut)
		FileCreateShortcut(@ScriptFullPath, $s_startup_shortcut)
		TrayItemSetState($startup_item, $TRAY_CHECKED)
	EndIf
EndFunc

Func ToggleAutoResizeSetting()
	If $b_auto_resize == 1 Then
    ; Disable the Auto-Resize!
    $b_auto_resize = 0
		TrayItemSetState($auto_resize_item, $TRAY_UNCHECKED)
	Else
    ; Enable the Auto-Resize!
    $b_auto_resize = 1
		TrayItemSetState($auto_resize_item, $TRAY_CHECKED)
	EndIf
	; Update the Registry with the new setting
  RegWrite("HKEY_LOCAL_MACHINE\SOFTWARE\WDSToggle","AutoResize","REG_SZ",$b_auto_resize)
EndFunc

Func ChangeToggleKey()
  Local $s_old_toggle_key
  $s_old_toggle_key = $s_toggle_key
	$s_toggle_key = InputBox("Set the WDS toogle key", _
		"Please, type the key that you want to use to toggle WDS." & @CR & @CR _
		& "You can set any combination of keys, including the Function keys." & @CR & @CR _
		& "Regular keys can be input directly. However special keys MUST be put between BRACKETS." & @CR & @CR _
		& "For instance, to set the toggle key to F8 you must input {F8}." & @CR & @CR _
		& "Finally, to use CTRL begin the key combination with ^, for ALT use ! and for SHIFT use +." & @CR & @CR _
		& "For example type ^g to set the Toggle Key combination to Ctrl+g, !t for Alt+t or !+t for Ctrl+Shift+t", _
		$s_toggle_key, "", 500, 400)
	If @Error <> 0 Then
		Return
	EndIf
	
	RegWrite("HKEY_LOCAL_MACHINE\SOFTWARE\WDSToggle","ToggleKey","REG_SZ",$s_toggle_key)
	TrayItemSetText($togglekey_item,"Change Toggle Key combo (currently set to '" & $s_toggle_key & "')")
	; The only means of updating the hotkey selection is to restart the program!
	
	; Remove the previous hotkey
	HotkeySet($s_old_toggle_key)
	; And set the new one
	HotkeySet($s_toggle_key, "WDSToggle")
	
	;If @Compiled Then
	;	Run(@ScriptFullPath)
	;Else
	;	Run(@AutoItExe & ' "' & @ScriptFullPath & '"')
	;EndIf
	;Exit 0
EndFunc

Func ExitProgram()
	Exit 0
EndFunc
